@@ -1,6 +1,6 @@
 /* ncdc - NCurses Direct Connect client
 
-  Copyright (c) 2011-2013 Yoran Heling
+  Copyright (c) 2011-2014 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -27,46 +27,6 @@
 #include "ncdc.h"
 #include "cc.h"
 
-
-// List (well, table) of users who are granted a slot. Key = UID (g_memdup'ed),
-// value = (void *)1. cc_init_global() is responsible for initializing it, but
-// the list is otherwise managed in commands.c.
-GHashTable *cc_granted = NULL;
-
-void cc_grant(hub_user_t *u) {
-  if(!g_hash_table_lookup(cc_granted, &u->uid))
-    g_hash_table_insert(cc_granted, g_memdup(&u->uid, 8), (void *)1);
-}
-
-
-// For use with qsort()
-static int cc_grant_cmp(const void *pa, const void *pb) {
-  guint64 a = *((guint64 *)pa);
-  guint64 b = *((guint64 *)pb);
-  hub_user_t *ua = g_hash_table_lookup(hub_uids, &a);
-  hub_user_t *ub = g_hash_table_lookup(hub_uids, &b);
-  return !ua && !ub ? (a > b ? 1 : a < b ? -1 : 0) :
-     ua && !ub ? 1 : !ua && ub ? -1 : g_utf8_collate(ua->name, ub->name);
-}
-
-
-// Get an ordered list of granted users. Must be g_free()'d after user.
-guint64 *cc_grant_list() {
-  guint64 *list = g_new(guint64, g_hash_table_size(cc_granted)+1);
-  guint64 *key;
-  GHashTableIter iter;
-  g_hash_table_iter_init(&iter, cc_granted);
-  int n = 0;
-  while(g_hash_table_iter_next(&iter, (gpointer *)&key, NULL))
-    list[n++] = *key;
-  list[n] = 0;
-  qsort(list, n, 8, cc_grant_cmp);
-  return list;
-}
-
-
-
-
 // List of expected incoming or outgoing connections.  This is list managed by
 // the functions below, in addition to cc_init_global() and cc_remove_hub(),
 
@@ -75,7 +35,6 @@ typedef struct cc_expect_t {
   char *nick;   // NMDC, hub encoding. Also set on ADC, but only for debugging purposes
   guint64 uid;
   guint16 port;
-  char cid[8];  // ADC
   char *token;  // ADC
   char *kp;     // ADC - slice-alloc'ed with 32 bytes
   time_t added;
@@ -120,10 +79,9 @@ void cc_expect_add(hub_t *hub, hub_user_t *u, guint16 port, char *t, gboolean dl
   e->dl = dl;
   e->uid = u->uid;
   e->port = port;
-  if(e->adc) {
+  if(e->adc)
     e->nick = g_strdup(u->name);
-    memcpy(e->cid, u->cid, 8);
-  } else
+  else
     e->nick = g_strdup(u->name_hub);
   if(u->kp) {
     e->kp = g_slice_alloc(32);
@@ -138,13 +96,16 @@ void cc_expect_add(hub_t *hub, hub_user_t *u, guint16 port, char *t, gboolean dl
 
 
 // Checks the expects list for the current connection, sets cc->dl, cc->uid,
-// cc->hub and cc->kp_user and removes it from the expects list. cc->cid and
-// cc->token must be known.
-static gboolean cc_expect_adc_rm(cc_t *cc) {
+// cc->hub and cc->kp_user and removes it from the expects list. cc->token must
+// be known, and either cc->cid must be set or a uid must be given.
+static gboolean cc_expect_adc_rm(cc_t *cc, guint64 uid) {
+  // We're calculating the uid for each (expect->hub->id, cc->cid) pair in the
+  // list and compare it with expect->uid to see if we've got the right user.
+  // This isn't the most efficient solution...
   GList *n;
   for(n=cc_expected->head; n; n=n->next) {
     cc_expect_t *e = n->data;
-    if(e->adc && e->port == cc->port && memcmp(cc->cid, e->cid, 8) == 0 && strcmp(cc->token, e->token) == 0) {
+    if(e->adc && e->port == cc->port && strcmp(cc->token, e->token) == 0 && e->uid == (cc->cid ? hub_user_adc_id(e->hub->id, cc->cid) : uid)) {
       cc->uid = e->uid;
       cc->hub = e->hub;
       cc->kp_user = e->kp;
@@ -286,12 +247,12 @@ struct cc_t {
   guint16 port;
   guint16 state;
   int dir;        // (NMDC) our direction. -1 = Upload, otherwise: Download $dir
-  char cid[24];   // (ADC) only the first 8 bytes are used for checking,
-                  // but the full 24 bytes are stored after receiving CINF (for logging)
+  char *cid;      // (ADC) base32-encoded CID
   int timeout_src;
   char remoteaddr[64]; // xxx.xxx.xxx.xxx:ppppp or [ipv6addr]:ppppp
   char *token;    // (ADC)
   char *last_file;
+  dlfile_thread_t *dlthread;
   guint64 uid;
   guint64 last_size;
   guint64 last_offset;
@@ -378,7 +339,6 @@ GSequence *cc_list;
 void cc_global_init() {
   cc_expected = g_queue_new();
   cc_list = g_sequence_new(NULL);
-  cc_granted = g_hash_table_new_full(g_int64_hash, g_int64_equal, g_free, NULL);
 
   throttle_list = g_hash_table_new_full(throttle_hash, throttle_equal, NULL, throttle_free);
   g_timeout_add_seconds_full(G_PRIORITY_LOW, 600, throttle_purge, NULL, NULL);
@@ -483,12 +443,6 @@ static void xfer_log_add(cc_t *cc) {
   if(!log)
     log = logfile_create("transfers");
 
-  char cid[40] = {};
-  if(cc->adc)
-    base32_encode(cc->cid, cid);
-  else
-    strcpy(cid, "-");
-
   char tth[40] = {};
   if(strcmp(cc->last_file, "files.xml.bz2") == 0)
     strcpy(tth, "-");
@@ -501,13 +455,15 @@ static void xfer_log_add(cc_t *cc) {
   char *file = adc_escape(cc->last_file, FALSE);
 
   yuri_t uri;
-  g_return_if_fail(yuri_parse(cc->remoteaddr, &uri) == 0);
+  g_return_if_fail(yuri_parse_copy(cc->remoteaddr, &uri) == 0);
 
   char *msg = g_strdup_printf("%s %s %s %s %c %c %s %d %"G_GUINT64_FORMAT" %"G_GUINT64_FORMAT" %"G_GUINT64_FORMAT" %s",
-    cc->hub ? cc->hub->tab->name : cc->hub_name, cid, nick, uri.host, cc->dl ? 'd' : 'u',
+    cc->hub ? cc->hub->tab->name : cc->hub_name, cc->adc ? cc->cid : "-", nick, uri.host, cc->dl ? 'd' : 'u',
     transfer_size == cc->last_length ? 'c' : 'i', tth, (int)(time(NULL)-cc->last_start),
     cc->last_size, cc->last_offset, transfer_size, file);
   logfile_add(log, msg);
+
+  free(uri.buf);
   g_free(msg);
   g_free(nick);
   g_free(file);
@@ -560,6 +516,7 @@ void cc_download(cc_t *cc, dl_t *dl) {
   g_return_if_fail(cc->dl && cc->state == CCS_IDLE);
 
   memcpy(cc->last_hash, dl->hash, 24);
+
   // get virtual path
   char fn[45] = {};
   if(dl->islist)
@@ -575,18 +532,29 @@ void cc_download(cc_t *cc, dl_t *dl) {
       net_writef(cc->net, "CGET tthl %s 0 -1\n", fn);
     else
       net_writef(cc->net, "$ADCGET tthl %s 0 -1|", fn);
+    cc->last_offset = 0;
+
   // otherwise, send GET request
   } else {
-    // Don't download files in chunks >= 2GB, older ncdc's don't like that.
-    int len = dl->islist ? -1 : MIN(G_MAXINT-1, dl->size-dl->have);
+    /* TODO: A more long-term rate calculation algorithm might be more suitable here */
+    cc->dlthread = dlfile_getchunk(dl, cc->uid, ratecalc_rate(net_rate_in(cc->net)));
+    if(!cc->dlthread) {
+      g_set_error_literal(&cc->err, 1, 0, "Download interrupted.");
+      cc_disconnect(cc, FALSE);
+      return;
+    }
+    cc->last_offset = ((guint64)cc->dlthread->chunk * DLFILE_CHUNKSIZE) + cc->dlthread->len;
+    gint64 len = dl->islist ? -1 :
+      MIN(((gint64)cc->dlthread->allocated * DLFILE_CHUNKSIZE) - cc->dlthread->len, (gint64)(dl->size - cc->last_offset));
+
     if(cc->adc)
-      net_writef(cc->net, "CGET file %s %"G_GUINT64_FORMAT" %d\n", fn, dl->have, len);
+      net_writef(cc->net, "CGET file %s %"G_GUINT64_FORMAT" %"G_GINT64_FORMAT"\n", fn, cc->last_offset, len);
     else
-      net_writef(cc->net, "$ADCGET file %s %"G_GUINT64_FORMAT" %d|", fn, dl->have, len);
+      net_writef(cc->net, "$ADCGET file %s %"G_GUINT64_FORMAT" %"G_GINT64_FORMAT"|", fn, cc->last_offset, len);
   }
+
   g_free(cc->last_file);
   cc->last_file = g_strdup(dl->islist ? "files.xml.bz2" : dl->dest);
-  cc->last_offset = dl->have;
   cc->last_size = dl->size;
   cc->last_length = 0; // to be filled in handle_adcsnd()
   cc->state = CCS_TRANSFER;
@@ -594,8 +562,6 @@ void cc_download(cc_t *cc, dl_t *dl) {
 
 
 static void handle_recvdone(net_t *n, void *dat) {
-  // Notify dl
-  dl_recv_done(dat);
   // If the connection is still active, log the transfer and check for more
   // stuff to download
   if(n && net_is_connected(n)) {
@@ -605,6 +571,8 @@ static void handle_recvdone(net_t *n, void *dat) {
     cc->state = CCS_IDLE;
     dl_user_cc(cc->uid, cc);
   }
+  // Notify dl
+  dlfile_recv_done(dat);
 }
 
 
@@ -624,34 +592,21 @@ static void handle_recvtth(net_t *n, char *buf, int read) {
 
 static void handle_adcsnd(cc_t *cc, gboolean tthl, guint64 start, gint64 bytes) {
   dl_t *dl = g_hash_table_lookup(dl_queue, cc->last_hash);
-  if(!dl) {
+  if(!dl || (!tthl && !cc->dlthread)) {
     g_set_error_literal(&cc->err, 1, 0, "Download interrupted.");
     cc_disconnect(cc, FALSE);
     return;
   }
-  // Some buggy clients (DCGUI) return $ADCSND with bytes = -1. They probably
-  // mean "all of the remaining bytes of the file" with that.
-  if(bytes < 0) {
-    if(tthl) {
-      g_set_error_literal(&cc->err, 1, 0, "Protocol error.");
-      cc_disconnect(cc, TRUE);
-      return;
-    }
-    bytes = cc->last_size - cc->last_offset;
-  }
 
   cc->last_length = bytes;
   cc->last_tthl = tthl;
   if(!tthl) {
-    g_return_if_fail(dl->have == start);
-    if(!dl->size)
+    if(dl->islist) {
       cc->last_size = dl->size = bytes;
-    void *ctx = dl_recv_create(cc->uid, cc->last_hash);
-    if(!ctx) {
-      g_set_error_literal(&cc->err, 1, 0, "Download interrupted.");
-      cc_disconnect(cc, FALSE);
-    } else
-      net_recvfile(cc->net, bytes, dl_recv_data, handle_recvdone, ctx);
+      dl->hassize = TRUE;
+    }
+    net_recvfile(cc->net, bytes, dlfile_recv, handle_recvdone, cc->dlthread);
+    cc->dlthread = NULL;
   } else {
     g_return_if_fail(start == 0 && bytes > 0 && (bytes%24) == 0 && bytes < 48*1024);
     net_readbytes(cc->net, bytes, handle_recvtth);
@@ -698,7 +653,7 @@ static void handle_adcget(cc_t *cc, char *type, char *id, guint64 start, gint64
     char *dat = db_fl_gettthl(root, &len);
     if(!dat)
       g_set_error_literal(err, 1, 51, "File Not Available");
-    else if(throttle_check(cc, root, G_MAXUINT64)) {
+    else if(!cc->slot_granted && throttle_check(cc, root, G_MAXUINT64)) {
       g_message("CC:%s: TTHL request throttled: %s", net_remoteaddr(cc->net), id);
       g_set_error_literal(err, 1, 50, "Action throttled");
     } else {
@@ -794,7 +749,7 @@ static void handle_adcget(cc_t *cc, char *type, char *id, guint64 start, gint64
   if(needslot && st.st_size < var_get_int(0, VAR_minislot_size))
     needslot = FALSE;
 
-  if(f && throttle_check(cc, f->tth, start)) {
+  if(f && !cc->slot_granted && throttle_check(cc, f->tth, start)) {
     g_message("CC:%s: File upload throttled: %s offset %"G_GUINT64_FORMAT, net_remoteaddr(cc->net), vpath, start);
     g_set_error_literal(err, 1, 50, "Action throttled");
     g_free(path);
@@ -839,9 +794,6 @@ static void handle_id(cc_t *cc, hub_user_t *u) {
 
   uit_conn_listchange(cc->iter, UITCONN_MOD);
 
-  if(cc->adc)
-    memcpy(cc->cid, u->cid, 8);
-
   // Set u->ip4 or u->ip6 if we didn't get this from the hub yet (NMDC, this
   // information is only used for display purposes).
   // Note that in the case of ADC, this function is called before the
@@ -851,11 +803,12 @@ static void handle_id(cc_t *cc, hub_user_t *u) {
   // first place).
   if(net_is_connected(cc->net) && (ip4_isany(u->ip4) && ip6_isany(u->ip6))) {
     yuri_t uri;
-    if(yuri_parse(net_remoteaddr(cc->net), &uri) == 0) {
-      if(yuri_validate_ipv4(uri.host, strlen(uri.host)) == 0)
+    if(yuri_parse_copy(net_remoteaddr(cc->net), &uri) == 0) {
+      if(uri.hosttype == YURI_IPV4)
         u->ip4 = ip4_pack(uri.host);
       else
         u->ip6 = ip6_pack(uri.host);
+      free(uri.buf);
     }
   }
 
@@ -871,7 +824,7 @@ static void handle_id(cc_t *cc, hub_user_t *u) {
     }
   }
 
-  cc->slot_granted = g_hash_table_lookup(cc_granted, &(u->uid)) ? TRUE : FALSE;
+  cc->slot_granted = db_users_get(u->hub->id, u->name) & DB_USERFLAG_GRANT ? TRUE : FALSE;
 }
 
 
@@ -894,7 +847,7 @@ static void adc_handle(net_t *net, char *msg, int _len) {
   }
 
   if(cmd.type != 'C') {
-    g_message("CC:%s: Not a client command: %s. --> %s", net_remoteaddr(cc->net), err->message, msg);
+    g_message("CC:%s: Not a client command: %s", net_remoteaddr(cc->net), msg);
     g_strfreev(cmd.argv);
     return;
   }
@@ -934,23 +887,20 @@ static void adc_handle(net_t *net, char *msg, int _len) {
       cc->state = CCS_IDLE;;
       char *id = adc_getparam(cmd.argv, "ID", NULL);
       char *token = adc_getparam(cmd.argv, "TO", NULL);
-      char cid[24];
-      if(istth(id))
-        base32_decode(id, cid);
       if(!id || (cc->active && !token)) {
         g_set_error_literal(&cc->err, 1, 0, "Protocol error.");
         g_message("CC:%s: No token or CID present: %s", net_remoteaddr(cc->net), msg);
         cc_disconnect(cc, TRUE);
         break;
-      } else if(!istth(id) || (!cc->active && memcmp(cid, cc->cid, 8) != 0)) {
+      } else if(!iscid(id) || (!cc->active && (!cc->hub || cc->uid != hub_user_adc_id(cc->hub->id, id)))) {
         g_set_error_literal(&cc->err, 1, 0, "Protocol error.");
         g_message("CC:%s: Incorrect CID: %s", net_remoteaddr(cc->net), msg);
         cc_disconnect(cc, TRUE);
         break;
       } else if(cc->active) {
         cc->token = g_strdup(token);
-        memcpy(cc->cid, cid, 24);
-        cc_expect_adc_rm(cc);
+        cc->cid = g_strdup(id);
+        cc_expect_adc_rm(cc, 0);
         hub_user_t *u = cc->uid ? g_hash_table_lookup(hub_uids, &cc->uid) : NULL;
         if(!u) {
           g_set_error_literal(&cc->err, 1, 0, "Protocol error.");
@@ -960,7 +910,7 @@ static void adc_handle(net_t *net, char *msg, int _len) {
         } else
           handle_id(cc, u);
       } else
-        memcpy(cc->cid, cid, 24);
+        cc->cid = g_strdup(id);
       // Perform keyprint validation
       // TODO: Throw an error if kp_user is set but we've not received a kp_real?
       if(cc->kp_real && cc->kp_user && memcmp(cc->kp_real, cc->kp_user, 32) != 0) {
@@ -1091,6 +1041,10 @@ static void adc_handle(net_t *net, char *msg, int _len) {
         dl_queue_setuerr(cc->uid, cc->last_hash, DLE_NOFILE, NULL);
         cc->state = CCS_IDLE;
         dl_user_cc(cc->uid, cc);
+        if(cc->dlthread) {
+          dlfile_recv_done(cc->dlthread);
+          cc->dlthread = NULL;
+        }
       }
 
     // Other message
@@ -1347,6 +1301,10 @@ static void nmdc_handle(net_t *net, char *cmd, int _len) {
       g_free(msg);
       cc->state = CCS_IDLE;
       dl_user_cc(cc->uid, cc);
+      if(cc->dlthread) {
+        dlfile_recv_done(cc->dlthread);
+        cc->dlthread = NULL;
+      }
     }
   }
   g_match_info_free(nfo);
@@ -1375,7 +1333,7 @@ static void handle_error(net_t *n, int action, const char *err) {
   if(net_is_disconnecting(n))
     net_disconnect(n);
   else
-    cc_disconnect(net_handle(n), action != NETERR_TIMEOUT);
+    cc_disconnect(net_handle(n), !net_is_asy(n) || action != NETERR_TIMEOUT);
 }
 
 
@@ -1441,7 +1399,7 @@ static void handle_connect(net_t *n, const char *addr) {
 
 void cc_nmdc_connect(cc_t *cc, const char *host, unsigned short port, const char *laddr, gboolean tls) {
   g_return_if_fail(cc->state == CCS_CONN);
-  g_snprintf(cc->remoteaddr, sizeof(cc->remoteaddr), yuri_validate_ipv6(host, strlen(host)) == 0 ? "[%s]:%d" : "%s:%d", host, (int)port);
+  g_snprintf(cc->remoteaddr, sizeof(cc->remoteaddr), ip6_isvalid(host) ? "[%s]:%d" : "%s:%d", host, (int)port);
   cc->tls = tls;
   net_connect(cc->net, host, port, laddr, handle_connect);
   g_clear_error(&cc->err);
@@ -1455,14 +1413,13 @@ void cc_adc_connect(cc_t *cc, hub_user_t *u, const char *laddr, unsigned short p
   cc->tls = tls;
   cc->adc = TRUE;
   cc->token = g_strdup(token);
-  memcpy(cc->cid, u->cid, 8);
   /* TODO: If the user has both ip4 and ip6, we should prefer the AF used to
    * connect to the hub, rather than ip4. */
   const char *host = !ip4_isany(u->ip4) ? ip4_unpack(u->ip4) : ip6_unpack(u->ip6);
   g_snprintf(cc->remoteaddr, sizeof(cc->remoteaddr), ip4_isany(u->ip4) ? "[%s]:%d" : "%s:%d", host, (int)port);
 
   // check whether this was as a reply to a RCM from us
-  cc_expect_adc_rm(cc);
+  cc_expect_adc_rm(cc, u->uid);
   if(!cc->kp_user && u->kp) {
     cc->kp_user = g_slice_alloc(32);
     memcpy(cc->kp_user, u->kp, 32);
@@ -1520,6 +1477,12 @@ static gboolean handle_timeout(gpointer dat) {
 
 void cc_disconnect(cc_t *cc, gboolean force) {
   g_return_if_fail(cc->state != CCS_DISCONN);
+  if(cc->dl && cc->uid)
+    dl_user_cc(cc->uid, NULL);
+  if(cc->dlthread) {
+    dlfile_recv_done(cc->dlthread);
+    cc->dlthread = NULL;
+  }
   if(cc->state == CCS_TRANSFER)
     xfer_log_add(cc);
   if(force || !net_is_asy(cc->net))
@@ -1530,8 +1493,6 @@ void cc_disconnect(cc_t *cc, gboolean force) {
   g_free(cc->token);
   cc->token = NULL;
   cc->state = CCS_DISCONN;
-  if(cc->dl && cc->uid)
-    dl_user_cc(cc->uid, NULL);
 }
 
 
@@ -1542,6 +1503,7 @@ void cc_free(cc_t *cc) {
     g_source_remove(cc->timeout_src);
   uit_conn_listchange(cc->iter, UITCONN_DEL);
   g_sequence_remove(cc->iter);
+  net_disconnect(cc->net);
   net_unref(cc->net);
   if(cc->err)
     g_error_free(cc->err);
@@ -1553,5 +1515,6 @@ void cc_free(cc_t *cc) {
   g_free(cc->nick);
   g_free(cc->hub_name);
   g_free(cc->last_file);
+  g_free(cc->cid);
   g_free(cc);
 }
