@@ -1,6 +1,6 @@
 /* ncdc - NCurses Direct Connect client
 
-  Copyright (c) 2011-2013 Yoran Heling
+  Copyright (c) 2011-2014 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -100,14 +100,14 @@ struct dl_user_t {
 struct dl_t {
   gboolean islist : 1;
   gboolean hastthl : 1;
-  gboolean active : 1;   // Whether it is being downloaded by someone
   gboolean flopen : 1;   // For lists: Whether to open a browse tab after completed download
   gboolean flmatch : 1;  // For lists: Whether to match queue after completed download
-  gboolean dlthread : 1; // Whether a dl thread is active
-  gboolean delete : 1;   // Pending delection
+  gboolean hassize : 1;  // For lists: Whether the size of the file list is known and validated
+  gboolean allbusy : 1;  // When no more unallocated blocks are available (maintained by dlfile.c)
   signed char prio;      // DLP_*
   char error;            // DLE_*
-  int incfd;             // file descriptor for this file in <incoming_dir>
+  unsigned char active_threads; // number of active downloading threads (maintained by dlfile.c)
+  int incfd;             // file descriptor for this file in <incoming_dir> (maintained by dlfile.c)
   char *error_msg;       // if error != DLE_NONE
   char *flsel;           // path to file/dir to select for filelists
   ui_tab_t *flpar;       // parent of the file list browser tab for filelists (might be a dangling pointer!)
@@ -115,11 +115,20 @@ struct dl_t {
   GPtrArray *u;          // list of users who have this file (GSequenceIter pointers into dl_user.queue)
   guint64 size;          // total size of the file
   guint64 have;          // what we have so far
+  guint64 hash_block;    // number of bytes that each block represents
   char *inc;             // path to the incomplete file (<incoming_dir>/<base32-hash>)
   char *dest;            // destination path
-  guint64 hash_block;    // number of bytes that each block represents
-  tth_ctx_t *hash_tth;   // TTH state of the last block that we have
   GSequenceIter *iter;   // used by ui_dl
+  GSList *threads;       // maintained by dlfile.c
+  guint8 *bitmap;        // Only used if hastthl, maintained by dlfile.c
+  guint bitmap_src;      // timeout source for flushing the bitmap, maintained by dlfile.c
+  /* Maintained by dlfile.c, protects dl_t.{have,bitmap,bitmap_src} and
+   * dlfile_thread_t.{allocated,avail,chunk}.
+   * Some other fields are shared, too, but those are never modified while a
+   * downloading thread is active and thus do not need synchronisation.  These
+   * include dl_t.{size,islist,hash,hash_block,incfd} and possibly more.
+   * TODO: dl.have isn't always protected yet! */
+  GStaticMutex lock;
 };
 
 #endif
@@ -166,7 +175,6 @@ char *dl_strerror(char err, const char *sub) {
 // dl_user_t related functions
 
 static gboolean dl_user_waitdone(gpointer dat);
-static void dl_queue_checkrm(dl_t *dl, gboolean justfin);
 
 
 // Determine whether a dl_user_dl struct can be considered as "enabled".
@@ -216,9 +224,7 @@ static dl_user_dl_t *dl_user_getdl(const dl_user_t *du) {
   GSequenceIter *i = g_sequence_get_begin_iter(du->queue);
   for(; !g_sequence_iter_is_end(i); i=g_sequence_iter_next(i)) {
     dl_user_dl_t *dud = g_sequence_get(i);
-    if(!dl_user_dl_enabled(dud))
-      break;
-    if(!dud->dl->active)
+    if(dl_user_dl_enabled(dud) && !dud->dl->allbusy)
       return dud;
   }
   return NULL;
@@ -236,15 +242,9 @@ static void dl_user_setstate(dl_user_t *du, int state) {
   else if(state >= 0 && du->state == DLU_WAI && state != DLU_WAI)
     g_source_remove(du->timeout);
 
-  // Update dl_user_dl.active, dl.active and dl_user.active if we came from the
-  // ACT state. These are set in dl_queue_start_user().
   // ACT -> x
-  if(state >= 0 && du->state == DLU_ACT && state != DLU_ACT && du->active) {
-    dl_user_dl_t *dud = du->active;
+  if(state >= 0 && du->state == DLU_ACT && state != DLU_ACT && du->active)
     du->active = NULL;
-    dud->dl->active = FALSE;
-    dl_queue_checkrm(dud->dl, FALSE);
-  }
 
   // Set state
   //g_debug("dlu:%"G_GINT64_MODIFIER"x: %d -> %d (active = %s)", du->uid, du->state, state, du->active ? "true":"false");
@@ -334,19 +334,15 @@ static void dl_user_rm(dl_t *dl, int i) {
   dl_user_dl_t *dud = g_sequence_get(dudi);
   dl_user_t *du = dud->u;
 
-  // Make sure to disconnect the user and disable dl->active if we happened to
-  // be actively downloading the file from this user.
+  // Make sure to disconnect the user if we happened to be actively downloading
+  // the file from this user.
   if(du->active == dud) {
     cc_disconnect(du->cc, TRUE);
     du->active = NULL;
-    dl->active = FALSE;
-    // Note that cc_disconnect() immediately calls dl_user_cc(), causing
-    // dl->active to be reset anyway. I'm not sure whether it's a good idea to
-    // rely on that, however.
   }
 
   uit_dl_dud_listchange(dud, UITDL_DEL);
-  g_sequence_remove(dudi); // dl_user_dl_free() will be called implicitely
+  g_sequence_remove(dudi); // dl_user_dl_free() will be called implicitly
   g_ptr_array_remove_index_fast(dl->u, i);
   dl_user_setstate(du, -1);
 }
@@ -400,19 +396,7 @@ static gboolean dl_queue_start_user(dl_user_t *du) {
   dl_t *dl = dud->dl;
   g_debug("dl:%016"G_GINT64_MODIFIER"x: using connection for %s", du->uid, dl->dest);
 
-  // For filelists: Don't allow resuming of the download. It could happen that
-  // the client modifies its filelist in between our retries. In that case the
-  // downloaded filelist would end up being corrupted. To avoid that: make sure
-  // lists are downloaded in one go, and throw away any incomplete data.
-  if(dl->islist && dl->have > 0) {
-    dl->have = dl->size = 0;
-    g_return_val_if_fail(close(dl->incfd) == 0, FALSE);
-    dl->incfd = open(dl->inc, O_WRONLY|O_CREAT|O_TRUNC, 0666);
-    g_return_val_if_fail(dl->incfd >= 0, FALSE);
-  }
-
   // Update state and connect
-  dl->active = TRUE;
   du->active = dud;
   dl_user_setstate(du, DLU_ACT);
   cc_download(du->cc, dl);
@@ -553,6 +537,7 @@ void dl_queue_addlist(hub_user_t *u, const char *sel, ui_tab_t *parent, gboolean
   g_return_if_fail(u && u->hasinfo);
   dl_t *dl = g_slice_new0(dl_t);
   dl->islist = TRUE;
+  g_static_mutex_init(&dl->lock);
   if(sel)
     dl->flsel = g_strdup(sel);
   dl->flpar = parent;
@@ -591,6 +576,7 @@ static gboolean dl_queue_addfile(guint64 uid, char *hash, guint64 size, char *fn
   if(g_hash_table_lookup(dl_queue, hash))
     return FALSE;
   dl_t *dl = g_slice_new0(dl_t);
+  g_static_mutex_init(&dl->lock);
   memcpy(dl->hash, hash, 24);
   dl->size = size;
   // Figure out dl->dest
@@ -600,6 +586,9 @@ static gboolean dl_queue_addfile(guint64 uid, char *hash, guint64 size, char *fn
   dl_queue_insert(dl, FALSE);
   dl_user_add(dl, uid, 0, NULL);
   db_dl_adduser(dl->hash, uid, 0, NULL);
+  // Empty files get a shortcut here
+  if(!size)
+    dlfile_finished(dl);
   return TRUE;
 }
 
@@ -690,38 +679,28 @@ int dl_queue_match_fl(guint64 uid, fl_list_t *fl, int *added) {
 
 // removes an item from the queue
 void dl_queue_rm(dl_t *dl) {
-  dl->delete = TRUE;
-
   // remove from the user info (this will also force a disconnect if the item
   // is being downloaded.)
   while(dl->u->len > 0)
     dl_user_rm(dl, 0);
-  // remove from dl list, if it's still in there. (Could have been removed
-  // before, while dlthread was active)
+  // remove from dl list
   if(g_hash_table_lookup(dl_queue, dl->hash)) {
     uit_dl_listchange(dl, UITDL_DEL);
     g_hash_table_remove(dl_queue, dl->hash);
   }
 
-  // Don't do anything else if the dlthread is still active. Wait until the
-  // thread stops and calls this function again to actually free and remove the
-  // stuff.
-  if(dl->dlthread)
+  // Don't do anything else if there is still an active downloading thread.
+  // Wait until all threads stop this function is called again to actually free
+  // and remove the stuff.
+  if(dl->active_threads)
     return;
 
   // remove from the database
   if(!dl->islist)
     db_dl_rm(dl->hash);
-  // close the incomplete file, in case it's still open
-  if(dl->incfd > 0)
-    g_warn_if_fail(close(dl->incfd) == 0);
-  // remove the incomplete file, in case we still have it
-  if(dl->inc && g_file_test(dl->inc, G_FILE_TEST_EXISTS))
-    unlink(dl->inc);
   // free and remove dl struct
   // and free
-  if(dl->hash_tth)
-    g_slice_free(tth_ctx_t, dl->hash_tth);
+  dlfile_rm(dl);
   g_ptr_array_unref(dl->u);
   g_free(dl->inc);
   g_free(dl->flsel);
@@ -731,43 +710,6 @@ void dl_queue_rm(dl_t *dl) {
 }
 
 
-// Called when dl->active is reset or when the download has finished. If both
-// actions are satisfied, we can _rm() the dl item. This makes sure that a dl
-// struct is not removed while active is true. Otherwise the user will be
-// forcibly disconnected in dl_user_rm(). (Which is what you want if you remove
-// it while downloading, not if it's removed after finishing the download).
-static void dl_queue_checkrm(dl_t *dl, gboolean justfin) {
-  if(dl->delete)
-    return;
-
-  if(justfin) {
-    g_return_if_fail(!dl->dlthread);
-    // If the download just finished, we might as well remove it from the
-    // database immediately. Makes sure we won't load it on the next startup.
-    if(!dl->islist)
-      db_dl_rm(dl->hash);
-
-    // Since the dl item is now considered as "disabled" by the download
-    // management code, make sure it is also last in every user's download
-    // queue. For performance reasons, we can also already remove the users who
-    // aren't active.
-    int i = 0;
-    while(i<dl->u->len) {
-      GSequenceIter *iter = g_ptr_array_index(dl->u, i);
-      dl_user_dl_t *dud = g_sequence_get(iter);
-      if(dud != dud->u->active)
-        dl_user_rm(dl, i);
-      else {
-        g_sequence_sort_changed(iter, dl_user_dl_sort, NULL);
-        i++;
-      }
-    }
-  }
-  if(!dl->active && !dl->dlthread && (dl->size || !dl->islist) && dl->have == dl->size)
-    dl_queue_rm(dl);
-}
-
-
 void dl_queue_setprio(dl_t *dl, signed char prio) {
   gboolean enabled = dl->prio <= DLP_OFF && prio > DLP_OFF;
   dl->prio = prio;
@@ -776,20 +718,25 @@ void dl_queue_setprio(dl_t *dl, signed char prio) {
   int i;
   for(i=0; i<dl->u->len; i++)
     g_sequence_sort_changed(g_ptr_array_index(dl->u, i), dl_user_dl_sort, NULL);
-  // Start the download if it is enabled
-  if(enabled)
-    dl_queue_start();
+  // Start downloading or re-attempt finalization if it is enabled
+  if(enabled) {
+    if(!dl->active_threads && (dl->hassize || !dl->islist) && dl->have == dl->size)
+      dlfile_finished(dl);
+    else
+      dl_queue_start();
+  }
+  /* TODO: Disconnect active users if the dl item is disabled */
 }
 
 
-#define dl_queue_seterr(dl, e, sub) do {\
-    (dl)->error = e;\
-    g_free((dl)->error_msg);\
-    (dl)->error_msg = (sub) ? g_strdup(sub) : NULL;\
-    dl_queue_setprio(dl, DLP_ERR);\
-    g_debug("Download of `%s' failed: %s", (dl)->dest, dl_strerror(e, sub));\
-    ui_mf(uit_main_tab, 0, "Download of `%s' failed: %s", (dl)->dest, dl_strerror(e, sub));\
-  } while(0)
+void dl_queue_seterr(dl_t *dl, char e, const char *sub) {
+  dl->error = e;
+  g_free(dl->error_msg);
+  dl->error_msg = sub ? g_strdup(sub) : NULL;
+  dl_queue_setprio(dl, DLP_ERR);
+  g_debug("Download of `%s' failed: %s", dl->dest, dl_strerror(e, sub));
+  ui_mf(uit_main_tab, 0, "Download of `%s' failed: %s", dl->dest, dl_strerror(e, sub));
+}
 
 
 // Set a user-specific error. If tth = NULL, the error will be set for all
@@ -797,7 +744,7 @@ void dl_queue_setprio(dl_t *dl, signed char prio) {
 void dl_queue_setuerr(guint64 uid, char *tth, char e, const char *emsg) {
   dl_t *dl = tth ? g_hash_table_lookup(dl_queue, tth) : NULL;
   dl_user_t *du = g_hash_table_lookup(queue_users, &uid);
-  if(!dl || (tth && !du))
+  if(!du || (tth && !dl))
     return;
 
   g_debug("%016"G_GINT64_MODIFIER"x: Setting download error for `%s' to: %s", uid, dl?dl->dest:"all", dl_strerror(e, emsg));
@@ -894,50 +841,8 @@ void dl_queue_rmuser(guint64 uid, char *tth) {
 // Managing of active downloads
 
 // Called when we've got a complete file
-static void dl_finished(dl_t *dl) {
+void dl_finished(dl_t *dl) {
   g_debug("dl: download of `%s' finished, removing from queue", dl->dest);
-  // close
-  if(dl->incfd > 0)
-    g_warn_if_fail(close(dl->incfd) == 0);
-  dl->incfd = 0;
-
-  char *fdest = g_filename_from_utf8(dl->dest, -1, NULL, NULL, NULL);
-  if(!fdest) // Just insist on UTF-8 if the conversion fails.
-    fdest = g_strdup(dl->dest);
-
-  // Create destination directory, if it does not exist yet.
-  char *parent = g_path_get_dirname(fdest);
-  if(g_mkdir_with_parents(parent, 0755) < 0)
-    dl_queue_seterr(dl, DLE_IO_DEST, g_strerror(errno));
-  g_free(parent);
-
-  // Prevent overwiting other files by appending a prefix to the destination if
-  // it already exists. It is assumed that fn + any dupe-prevention-extension
-  // does not exceed NAME_MAX. (Not that checking against NAME_MAX is really
-  // reliable - some filesystems have an even more strict limit)
-  int num = 1;
-  char *dest = g_strdup(fdest);
-  while(!dl->islist && g_file_test(dest, G_FILE_TEST_EXISTS)) {
-    g_free(dest);
-    dest = g_strdup_printf("%s.%d", fdest, num++);
-  }
-
-  // Move the file to the destination.
-  // TODO: this may block for a while if they are not on the same filesystem,
-  // do this in a separate thread?
-  GError *err = NULL;
-  if(dl->prio != DLP_ERR) {
-    file_move(dl->inc, dest, dl->islist, &err);
-    if(err) {
-      // Note: 'dest' is in filename encoding, but that's not a huge problem
-      // when it's just going to a g_warning().
-      g_warning("Error moving `%s' to `%s': %s", dl->inc, dest, err->message);
-      dl_queue_seterr(dl, DLE_IO_DEST, err->message);
-      g_error_free(err);
-    }
-  }
-  g_free(dest);
-  g_free(fdest);
 
   // open the file list
   if(dl->islist && dl->prio != DLP_ERR) {
@@ -948,8 +853,8 @@ static void dl_finished(dl_t *dl) {
         FALSE, dl->flsel, dl->flpar, dl->flopen, dl->flmatch);
     ui_tab_cur = cur;
   }
-  // and check whether we can remove this item from the queue
-  dl_queue_checkrm(dl, TRUE);
+
+  dl_queue_rm(dl);
 }
 
 
@@ -962,11 +867,9 @@ void dl_settthl(guint64 uid, char *tth, char *tthl, int len) {
     return;
   g_return_if_fail(du->state == DLU_ACT);
   g_return_if_fail(!dl->islist);
-  g_return_if_fail(!dl->have);
-  g_return_if_fail(!dl->dlthread);
-  // Ignore this if we already have the TTHL data. Currently, this situation
-  // can't happen, but it may be possible with segmented downloading.
-  g_return_if_fail(!dl->hastthl);
+  // We accidentally downloaded the TTHL from multiple users. Just discard this data.
+  if(dl->hastthl)
+    return;
 
   g_debug("dl:%016"G_GINT64_MODIFIER"x: Received TTHL data for %s (len = %d, bs = %"G_GUINT64_FORMAT")", uid, dl->dest, len, tth_blocksize(dl->size, len/24));
 
@@ -1002,240 +905,16 @@ void dl_settthl(guint64 uid, char *tth, char *tthl, int len) {
 
 
 
-// Data receive background thread
-
-// The background thread access the following dl_t members:
-// - size       (read only  - not changed in an other thread, no problem)
-// - have       (read/write - also read in other threads - TODO: this could be a problem)
-// - hash_block (read only  - not changed in an other thread, no problem)
-// - hash_tth   (read/write - not used in other threads, no problem)
-// - incfd      (read/write - not used in other threads, no problem)
-// - islist     (read only  - not changed in an other thread, no problem)
-
-typedef struct recv_ctx_t {
-  dl_t *dl;
-  guint64 uid;
-  char *err_msg, *uerr_msg;
-  char err, uerr;
-  fadv_t adv;
-} recv_ctx_t;
-
-
-void *dl_recv_create(guint64 uid, const char *tth) {
-  dl_t *dl = g_hash_table_lookup(dl_queue, tth);
-  dl_user_t *du = g_hash_table_lookup(queue_users, &uid);
-  if(!dl || !du)
-    return NULL;
-  g_return_val_if_fail(!dl->dlthread, NULL);
-  g_return_val_if_fail(du->state == DLU_ACT && du->active && du->active->dl == dl, NULL);
-  g_return_val_if_fail(dl->islist || dl->hastthl, NULL);
-
-  // open dl->incfd, if it's not open yet
-  if(dl->incfd <= 0) {
-    dl->incfd = open(dl->inc, O_WRONLY|O_CREAT, 0666);
-    if(dl->incfd < 0 || lseek(dl->incfd, dl->have, SEEK_SET) == (off_t)-1) {
-      g_warning("Error opening %s: %s", dl->inc, g_strerror(errno));
-      dl_queue_seterr(dl, DLE_IO_INC, g_strerror(errno));
-      return NULL;
-    }
-  }
-
-  // Create context and mark the dl item als being active
-  dl->dlthread = TRUE;
-  recv_ctx_t *c = g_slice_new0(recv_ctx_t);
-  c->uid = uid;
-  c->dl = dl;
-  fadv_init(&c->adv, dl->incfd, dl->have, VAR_FFC_DOWNLOAD);
-
-  return c;
-}
-
-
-void dl_recv_done(void *dat) {
-  recv_ctx_t *c = dat;
-
-  fadv_close(&c->adv);
-
-  // Indicate that the dl thread has stopped
-  c->dl->dlthread = FALSE;
-
-  if(c->dl->delete)
-    dl_queue_rm(c->dl);
-
-  else {
-    // Set error status if necessary
-    if(c->err)
-      dl_queue_seterr(c->dl, c->err, c->err_msg);
-    if(c->uerr)
-      dl_queue_setuerr(c->uid, c->dl->hash, c->uerr, c->uerr_msg);
-
-    if(c->dl->have >= c->dl->size) {
-      g_warn_if_fail(c->dl->have == c->dl->size);
-      dl_finished(c->dl);
-    }
-  }
-
-  // Clean up
-  g_free(c->uerr_msg);
-  g_free(c->err_msg);
-  g_slice_free(recv_ctx_t, c);
-}
-
-
-static gboolean dl_recv_check(dl_t *dl, int num, char *tth) {
-  // We don't have TTHL data for small files, so check against the root hash instead.
-  if(dl->size < dl->hash_block) {
-    g_return_val_if_fail(num == 0, FALSE);
-    return memcmp(tth, dl->hash, 24) == 0 ? TRUE : FALSE;
-  }
-  // Otherwise, check against the TTHL data in the database.
-  return db_dl_checkhash(dl->hash, num, tth);
-}
-
-
-// (Incrementally) hashes the incoming data and checks that what we have is
-// still correct. When this function is called, dl->length should refer to the
-// point where the newly received data will be written to.
-// Returns -1 if nothing went wrong, any other number to indicate which block
-// failed the hash check.
-static int dl_recv_update(dl_t *dl, const char *buf, int length) {
-  int block = dl->have / dl->hash_block;
-  guint64 cur = dl->have % dl->hash_block;
-
-  if(!dl->hash_tth) {
-    g_return_val_if_fail(!cur, FALSE);
-    dl->hash_tth = g_slice_new(tth_ctx_t);
-    tth_init(dl->hash_tth);
-  }
-
-  char tth[24];
-  while(length > 0) {
-    int w = MIN(dl->hash_block - cur, length);
-    tth_update(dl->hash_tth, buf, w);
-    length -= w;
-    cur += w;
-    buf += w;
-    // we have a complete block, validate it.
-    if(cur == dl->hash_block || (!length && dl->size == (block*dl->hash_block)+cur)) {
-      tth_final(dl->hash_tth, tth);
-      tth_init(dl->hash_tth);
-      if(!dl_recv_check(dl, block, tth))
-        return block;
-      cur = 0;
-      block++;
-    }
-  }
-
-  return -1;
-}
-
-
-// Called directly from net.c.
-gboolean dl_recv_data(void *dat, const char *buf, int length) {
-  recv_ctx_t *c = dat;
-
-  while(length > 0) {
-    // write
-    int r = write(c->dl->incfd, buf, length);
-    if(r < 0) {
-      c->err = DLE_IO_INC;
-      c->err_msg = g_strdup(g_strerror(errno));
-      return FALSE;
-    }
-    fadv_purge(&c->adv, r);
-
-    // check hash
-    int fail = c->dl->islist ? -1 : dl_recv_update(c->dl, buf, r);
-    if(fail >= 0) {
-      c->uerr = DLE_HASH;
-      c->uerr_msg = g_strdup_printf("Hash for block %d does not match.", fail);
-      // Delete the failed block and everything after it, so that a resume is possible.
-      c->dl->have = fail*c->dl->hash_block;
-      // I have no idea what to do when these functions fail. Resuming the
-      // download without an ncdc restart won't work if lseek() fails, resuming
-      // the download after an ncdc restart may result in a corrupted download
-      // if the ftruncate() fails. Either way, resuming isn't possible in a
-      // reliable fashion, so perhaps we should throw away the entire inc file?
-      off_t rs = lseek(c->dl->incfd, c->dl->have, SEEK_SET);
-      int rt = ftruncate(c->dl->incfd, c->dl->have);
-      if(rs == (off_t)-1 || rt == -1) {
-        g_warning("Error recovering from hash failure: %s", g_strerror(errno));
-        c->err = DLE_IO_INC;
-        c->err_msg = g_strdup(g_strerror(errno));
-      }
-      return FALSE;
-    }
-
-    // update vars
-    length -= r;
-    buf += r;
-    c->dl->have += r;
-  }
-
-  // TODO: if dl->have == d->size, close() the file here? Since close() may
-  // actually flush the data to disk and thus block for a while.
-
-  return TRUE;
-}
-
-
-
-
 
 // Loading/initializing the download queue on startup
 
 
-// Checks the incoming file for what we already have and modifies dl->have and
-// dl->hash_tth accordingly.
-void dl_load_partial(dl_t *dl) {
-  // get size of the incomplete file, but only if we have tthl info
-  char *fn = NULL;
-  if(dl->hastthl) {
-    char tth[40] = {};
-    base32_encode(dl->hash, tth);
-    fn = g_build_filename(var_get(0, VAR_incoming_dir), tth, NULL);
-    struct stat st;
-    if(stat(fn, &st) >= 0)
-      dl->have = st.st_size;
-  }
-
-  // If we have already downloaded some data, hash the last block and update
-  // dl->hash_tth.
-  guint64 left = dl->hash_block ? dl->have % dl->hash_block : 0;
-  if(left > 0) {
-    dl->have -= left;
-    int fd = open(fn, O_RDONLY);
-    if(fd < 0 || lseek(fd, dl->have, SEEK_SET) == (off_t)-1) {
-      g_warning("Error opening %s: %s. Throwing away last block.", fn, g_strerror(errno));
-      left = 0;
-      if(fd >= 0)
-        close(fd);
-    }
-    while(left > 0) {
-      char buf[10240];
-      int r = read(fd, buf, MIN(left, 10240));
-      if(r < 0) {
-        g_warning("Error reading from %s: %s. Throwing away unreadable data.", fn, g_strerror(errno));
-        left = 0;
-        break;
-      }
-      dl_recv_update(dl, buf, r);
-      dl->have += r;
-      left -= r;
-    }
-    if(fd >= 0)
-      close(fd);
-  }
-
-  g_free(fn);
-}
-
-
 // Creates and inserts a dl_t item from the database in the queue
 void dl_load_dl(const char *tth, guint64 size, const char *dest, signed char prio, char error, const char *error_msg, int tthllen) {
   g_return_if_fail(dest);
 
   dl_t *dl = g_slice_new0(dl_t);
+  g_static_mutex_init(&dl->lock);
   memcpy(dl->hash, tth, 24);
   dl->size = size;
   dl->prio = prio;
@@ -1274,7 +953,7 @@ void dl_init_global() {
   GHashTableIter iter;
   g_hash_table_iter_init(&iter, dl_queue);
   while(g_hash_table_iter_next(&iter, NULL, (gpointer *)&dl))
-    dl_load_partial(dl);
+    dlfile_load(dl);
   // Delete old filelists
   dl_fl_clean(NULL);
 }
