@@ -1,6 +1,6 @@
 /* ncdc - NCurses Direct Connect client
 
-  Copyright (c) 2011 Yoran Heling
+  Copyright (c) 2011-2014 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -90,10 +90,8 @@ static const doc_cmd_t doc_cmds[] = {
 },
 { "grant", "[-list|<user>]", "Grant someone a slot.",
   "Grant someone a slot. This allows the user to download from you even if you"
-  " have no free slots.  The slot will be granted for as long as ncdc stays"
-  " open or the /ungrant command is used. If you restart ncdc, the user will"
-  " have to wait for a regular slot. Unless, of course, you /grant a slot"
-  " again.\n\n"
+  " have no free slots.  The slot will remain granted until the /ungrant"
+  " command is used, even if ncdc has been restarted in the mean time.\n\n"
   "To get a list of users whom you have granted a slot, use `/grant' without"
   " arguments or with `-list'. Be warned that using `/grant' without arguments on"
   " a PM tab will grant the slot to the user you are talking with. Make sure to"
@@ -225,7 +223,7 @@ static const doc_cmd_t doc_cmds[] = {
   " you give it will be public. An initial `/refresh' is done automatically on"
   " the added directory."
 },
-{ "ungrant", "[<username>|<id>]", "Revoke a granted slot.",
+{ "ungrant", "[<user>]", "Revoke a granted slot.",
   NULL
 },
 { "unset", "[<key>]", "Unset a global configuration variable.",
@@ -384,7 +382,7 @@ static const doc_set_t doc_sets[] = {
   " files to their final destination."
 },
 { "download_exclude", 0, "<regex>",
-  "When recursively adding a directory to the download queue - by pressing `b'"
+  "When recursively adding a directory to the download queue - by pressing `d'"
   " on a directory in the file list browser - any item in the selected"
   " directory with a name that matches this regular expression will not be"
   " added to the download queue.\n\n"
@@ -398,6 +396,10 @@ static const doc_set_t doc_sets[] = {
   " TCP overhead are not counted towards this limit, so the actual bandwidth"
   " usage might be a little higher."
 },
+{ "download_segment", 0, "<size>",
+  "Minimum segment size to use when requesting file data from another user."
+  " Set to 0 to disable segmented downloading."
+},
 { "download_slots", 0, "<integer>",
   "Maximum number of simultaneous downloads."
 },
@@ -616,6 +618,10 @@ static const doc_set_t doc_sets[] = {
   "PgUp/PgDown  Select one page of items up/down.\n"\
   "End/Home     Select last/first item in the list.\n"
 
+#define SEARCH_KEYS \
+  "/            Start incremental regex search (press Return to stop editing).\n"\
+  ",/.          Search next / previous.\n"
+
 typedef struct doc_key_t {
   char const *sect, *title, *desc;
 } doc_key_t;
@@ -656,9 +662,10 @@ static const doc_key_t doc_keys[] = {
 },
 { "browse", "File browser",
   LISTING_KEYS
+  SEARCH_KEYS
   "Right/l      Open selected directory.\n"
   "Left/h       Open parent directory.\n"
-  "o            Toggle sorting directories before files.\n"
+  "t            Toggle sorting directories before files.\n"
   "s            Order by file size.\n"
   "n            Order by file name.\n"
   "d            Add selected file/directory to the download queue.\n"
@@ -711,6 +718,7 @@ static const doc_key_t doc_keys[] = {
 },
 { "userlist", "User list tab",
   LISTING_KEYS
+  SEARCH_KEYS
   "o            Toggle sorting OPs before others.\n"
   "s/S          Order by share size.\n"
   "u/U          Order by username.\n"
@@ -721,12 +729,14 @@ static const doc_key_t doc_keys[] = {
   "i/Return     Toggle information box.\n"
   "m            Send a PM to the selected user.\n"
   "g            Grant a slot to the selected user.\n"
-  "b/B          Browse the selected users' list, B to force a redownload."
+  "b/B          Browse the selected users' list, B to force a redownload.\n"
+  "q            Match selected users' list with the download queue."
 },
 
 { NULL }
 };
 
 #undef LISTING_KEYS
+#undef SEARCH_KEYS
 
 #endif // DOC_KEY
