@@ -1,6 +1,6 @@
 /* ncdc - NCurses Direct Connect client
 
-  Copyright (c) 2011-2013 Yoran Heling
+  Copyright (c) 2011-2014 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -56,7 +56,6 @@ struct hub_user_t {
   char *conn;     // NMDC: string pointer, ADC: GUINT_TO_POINTER() of the US param
   char *mail;
   char *client;
-  char cid[8];   // for ADC - only the first 8 bytes of the CID, for simple verification purposes
   guint64 uid;
   guint64 sharesize;
   char *kp;      // ADC with KEYP, 32 bytes slice-alloc'ed
@@ -138,30 +137,52 @@ GHashTable *hubs = NULL;
 
 // hub_user_t related functions
 
+/* Generate user ID for ADC */
+guint64 hub_user_adc_id(guint64 hubid, const char *cid) {
+  tiger_ctx_t t;
+  char tmp[MAX(MAXCIDLEN, 24)];
+  guint64 uid;
+
+  tiger_init(&t);
+  tiger_update(&t, (char *)&hubid, 8);
+  base32_decode(cid, tmp);
+  tiger_update(&t, tmp, (strlen(cid)*5)/8);
+  tiger_final(&t, tmp);
+  memcpy(&uid, tmp, 8);
+
+  return uid;
+}
+
+
+guint64 hub_user_nmdc_id(guint64 hubid, const char *name) {
+  tiger_ctx_t t;
+  guint64 uid;
+  char tmp[24];
+
+  tiger_init(&t);
+  tiger_update(&t, (char *)&hubid, 8);
+  tiger_update(&t, name, strlen(name));
+  tiger_final(&t, tmp);
+  memcpy(&uid, tmp, 8);
+  return uid;
+}
+
 
 // cid is required for ADC. expected to be base32-encoded.
 static hub_user_t *user_add(hub_t *hub, const char *name, const char *cid) {
   hub_user_t *u = g_hash_table_lookup(hub->users, name);
   if(u)
     return u;
-  tiger_ctx_t t;
-  char tmp[24];
-  tiger_init(&t);
-  tiger_update(&t, (char *)&(hub->id), 8);
   u = g_slice_new0(hub_user_t);
   u->hub = hub;
   if(hub->adc) {
     u->name = g_strdup(name);
-    base32_decode(cid, tmp);
-    memcpy(u->cid, tmp, 8);
-    tiger_update(&t, tmp, 24);
+    u->uid = hub_user_adc_id(hub->id, cid);
   } else {
     u->name_hub = g_strdup(name);
     u->name = charset_convert(hub, TRUE, name);
-    tiger_update(&t, u->name_hub, strlen(u->name_hub));
+    u->uid = hub_user_nmdc_id(hub->id, name);
   }
-  tiger_final(&t, tmp);
-  memcpy(&(u->uid), tmp, 8);
   // insert in hub->users
   g_hash_table_insert(hub->users, hub->adc ? u->name : u->name_hub, u);
   // insert in hub_uids
@@ -218,7 +239,7 @@ void hub_user_suggest(hub_t *hub, char *str, char **sug) {
   int i=0, len = strlen(str);
   g_hash_table_iter_init(&iter, hub->users);
   while(i<20 && g_hash_table_iter_next(&iter, NULL, (gpointer *)&u))
-    if(g_ascii_strncasecmp(u->name, str, len) == 0 && strlen(u->name) != len)
+    if(g_ascii_strncasecmp(u->name, str, len) == 0)
       sug[i++] = g_strdup(u->name);
   qsort(sug, i, sizeof(char *), cmpstringp);
 }
@@ -312,11 +333,13 @@ static void user_nmdc_nfo(hub_t *hub, hub_user_t *u, char *str) {
 
   // connection and flag
   str = next;
-  if(!(next = strchr(str, '$')) || str == next)
+  if(!(next = strchr(str, '$')))
     return;
-  flags = *(next-1);
-  *(next-1) = *next = 0; next++;
-
+  if(str != next) {
+    flags = *(next-1);
+    *(next-1) = 0;
+  }
+  *next = 0; next++;
   conn = str;
   cleanspace(conn);
 
@@ -750,7 +773,7 @@ void hub_send_nfo(hub_t *hub) {
     char *nconn = nmdc_encode_and_escape(hub, conn?conn:"0.005");
     char *nmail = nmdc_encode_and_escape(hub, mail?mail:"");
     nfo = g_strdup_printf("$MyINFO $ALL %s %s<ncdc V:%s,M:%c,H:%d/%d/%d,S:%d>$ $%s%c$%s$%"G_GUINT64_FORMAT"$|",
-      hub->nick_hub, ndesc, main_version, ip ? 'P' : 'A', h_norm, h_reg, h_op,
+      hub->nick_hub, ndesc, main_version, ip ? 'A' : 'P', h_norm, h_reg, h_op,
       slots, nconn, 1 | (sup_tls ? 0x10 : 0), nmail, share);
     g_free(ndesc);
     g_free(nconn);
@@ -844,22 +867,17 @@ static void setownip(hub_t *hub, hub_user_t *u) {
 }
 
 
-static void adc_sch_reply_send(hub_t *hub, const char *host, unsigned short port, GString *r, const char *key) {
-  if(!host) {
+static void adc_sch_reply_send(hub_t *hub, net_udp_t *udp, GString *r, const char *key) {
+  if(!udp) {
     net_writestr(hub->net, r->str);
     return;
   } else if(!key || var_get_int(0, VAR_sudp_policy) == VAR_SUDPP_DISABLE) {
-    net_udp_send(host, port, r->str);
+    net_udp_send(udp, r->str);
     return;
   }
 
   // net_udp_* can't log this since it will be encrypted, so log it here.
-  char end = r->len > 0 ? r->str[r->len-1] : 0;
-  if(end == '\n')
-    r->str[r->len-1] = 0;
-  g_debug("SUDP:%s:%d: %s", host, (int)port, r->str);
-  if(end == '\n')
-    r->str[r->len-1] = end;
+  g_debug("SUDP:%s> %.*s", udp->addr, (int)r->len-1, r->str);
 
   // prepend 16 random bytes to message
   char nonce[16];
@@ -874,7 +892,7 @@ static void adc_sch_reply_send(hub_t *hub, const char *host, unsigned short port
 
   // Now encrypt & send
   crypt_aes128cbc(TRUE, key, 16, r->str, r->len);
-  net_udp_send_raw(host, port, r->str, r->len);
+  net_udp_send_raw(udp, r->str, r->len);
 }
 
 
@@ -893,20 +911,23 @@ static void adc_sch_reply(hub_t *hub, adc_cmd_t *cmd, hub_user_t *u, fl_list_t *
 
   char tth[40] = {};
   char *cid = NULL;
-  char *host = NULL;
-  unsigned short port = 0;
+  net_udp_t udpbuf, *udp = NULL;
   if((u->hasudp4 && u->udp4) || (u->hasudp6 && u->udp6)) {
     cid = var_get(0, VAR_cid);
     /* TODO: If the user has both UDP4 and UDP6, we should prefer the AF used
      * to connect to the hub, rather than IPv4. */
-    host = g_strdup(u->hasudp4 && u->udp4 ? ip4_unpack(u->ip4) : ip6_unpack(u->ip6));
-    port = u->hasudp4 && u->udp4 ? u->udp4 : u->udp6;
+    udp = &udpbuf;
+    net_udp_init(udp,
+        u->hasudp4 && u->udp4 ? ip4_unpack(u->ip4) : ip6_unpack(u->ip6),
+        u->hasudp4 && u->udp4 ? u->udp4 : u->udp6,
+        var_get(hub->id, VAR_local_address)
+    );
   }
 
   int i;
   for(i=0; i<len; i++) {
-    GString *r = host ? adc_generate('U', ADCC_RES, 0, 0) : adc_generate('D', ADCC_RES, hub->sid, cmd->source);
-    if(host)
+    GString *r = udp ? adc_generate('U', ADCC_RES, 0, 0) : adc_generate('D', ADCC_RES, hub->sid, cmd->source);
+    if(udp)
       g_string_append_printf(r, " %s", cid);
     if(to)
       adc_append(r, "TO", to);
@@ -921,11 +942,12 @@ static void adc_sch_reply(hub_t *hub, adc_cmd_t *cmd, hub_user_t *u, fl_list_t *
       g_string_append_c(r, '/'); // make sure a directory path ends with a slash
     g_string_append_c(r, '\n');
 
-    adc_sch_reply_send(hub, host, port, r, ky ? sudpkey : NULL);
+    adc_sch_reply_send(hub, udp, r, ky ? sudpkey : NULL);
     g_string_free(r, TRUE);
   }
 
-  g_free(host);
+  if(udp)
+    net_udp_destroy(udp);
 }
 
 
@@ -1071,9 +1093,7 @@ static void adc_handle(net_t *net, char *msg, int _len) {
       if(!u) {
         char *nick = adc_getparam(cmd.argv, "NI", NULL);
         char *cid = adc_getparam(cmd.argv, "ID", NULL);
-        // Note that the ADC spec allows hashes of varying length. I'm limiting
-        // myself to TTH hashes here since that is more memory-efficient.
-        if(nick && cid && istth(cid))
+        if(nick && cid && iscid(cid))
           u = user_add(hub, nick, cid);
       }
       if(!u)
@@ -1134,8 +1154,15 @@ static void adc_handle(net_t *net, char *msg, int _len) {
     else {
       int code = (cmd.argv[0][1]-'0')*10 + (cmd.argv[0][2]-'0');
       int severity = cmd.argv[0][0]-'0';
-      ui_mf(hub->tab, UIP_LOW, "(%s-%02d) %s",
-        !severity ? "status" : severity == 1 ? "warning" : "error", code, cmd.argv[1]);
+      /* Direct STA messages are usually from CTM/RCM errors. Passing those to
+       * the main chat is spammy and unintuitive. */
+      if(cmd.type != 'D')
+        ui_mf(hub->tab, UIP_LOW, "(%s-%02d) %s",
+          !severity ? "status" : severity == 1 ? "warning" : "error", code, cmd.argv[1]);
+      else {
+        hub_user_t *u = g_hash_table_lookup(hub->sessions, GINT_TO_POINTER(cmd.source));
+        g_message("Direct Status message from %s on %s: %s", u ? u->name : "unknown user", net_remoteaddr(hub->net), msg);
+      }
     }
     break;
 
@@ -1144,7 +1171,7 @@ static void adc_handle(net_t *net, char *msg, int _len) {
       g_message("Invalid message from %s: %s", net_remoteaddr(hub->net), msg);
     else if(var_get_int(hub->id, VAR_tls_policy) == VAR_TLSP_DISABLE ? !is_adc_proto(cmd.argv[0]) : !is_valid_proto(cmd.argv[0])) {
       GString *r = adc_generate('D', ADCC_STA, hub->sid, cmd.source);
-      g_string_append(r, " 141 Unknown\\protocol");
+      g_string_append(r, " 141 Unknown\\sprotocol");
       adc_append(r, "PR", cmd.argv[0]);
       adc_append(r, "TO", cmd.argv[2]);
       g_string_append_c(r, '\n');
@@ -1348,6 +1375,10 @@ static void nmdc_search(hub_t *hub, char *from, unsigned short port, int size_m,
   char tth[44] = "TTH:";
   tth[43] = 0;
 
+  net_udp_t udp;
+  if(port)
+    net_udp_init(&udp, from, port, var_get(hub->id, VAR_local_address));
+
   while(--i>=0) {
     char *fl = fl_list_path(res[i]);
     // Windows style path delimiters... why!?
@@ -1366,12 +1397,15 @@ static void nmdc_search(hub_t *hub, char *from, unsigned short port, int size_m,
     if(!port)
       net_writef(hub->net, "%s\05%s|", msg, from);
     else
-      net_udp_sendf(from, port, "%s|", msg);
+      net_udp_sendf(&udp, "%s|", msg);
     g_free(fl);
     g_free(msg);
     g_free(size);
     g_free(tmp);
   }
+
+  if(port)
+    net_udp_destroy(&udp);
 }
 
 
@@ -1529,7 +1563,7 @@ static void nmdc_handle(net_t *net, char *cmd, int _len) {
         continue;
       *sep = 0;
       hub_user_t *u = user_add(hub, *cur, NULL);
-      if(yuri_validate_ipv4(sep+1, strlen(sep+1)) == 0) {
+      if(ip4_isvalid(sep+1)) {
         struct in_addr new = ip4_pack(sep+1);
         if(ip4_cmp(new, u->ip4) != 0) {
           u->ip4 = new;
@@ -1621,8 +1655,8 @@ static void nmdc_handle(net_t *net, char *cmd, int _len) {
       g_message("Received a $ConnectToMe for someone else (to %s from %s)", me, addr);
     else {
       yuri_t uri;
-      if(yuri_parse(addr, &uri) != 0 || uri.scheme[0] != 0 || uri.port == 0 ||
-          (yuri_validate_ipv4(uri.host, strlen(uri.host)) != 0 && yuri_validate_ipv6(uri.host, strlen(uri.host)) != 0))
+      if(yuri_parse(addr, &uri) != 0 || *uri.scheme || uri.port == 0 ||
+          uri.hosttype == YURI_DOMAIN || *uri.path || *uri.query || *uri.fragment)
         g_message("Invalid host:port in $ConnectToMe (%s)", addr);
       else
         cc_nmdc_connect(cc_create(hub), uri.host, uri.port, var_get(hub->id, VAR_local_address), *tls ? TRUE : FALSE);
@@ -1668,13 +1702,13 @@ static void nmdc_handle(net_t *net, char *cmd, int _len) {
     char *query = g_match_info_fetch(nfo, 6);
     unsigned short port = 0;
     char *nfrom = NULL;
-    yuri_t uri;
     if(strncmp(from, "Hub:", 4) == 0) {
       if(strcmp(from+4, hub->nick_hub) != 0) /* Not our nick */
         nfrom = from+4;
     } else {
-      if(yuri_parse(from, &uri) != 0 || uri.scheme[0] != 0 || uri.port == 0 ||
-          (yuri_validate_ipv4(uri.host, strlen(uri.host)) != 0 && yuri_validate_ipv6(uri.host, strlen(uri.host)) != 0))
+      yuri_t uri;
+      if(yuri_parse(from, &uri) != 0 || *uri.scheme || uri.port == 0 ||
+          uri.hosttype == YURI_DOMAIN || *uri.path || *uri.query || *uri.fragment)
         g_message("Invalid host:port in $Search (%s)", from);
       else if(!listen_hub_active(hub->id) || strcmp(uri.host, hub_ip(hub)) != 0 || uri.port != listen_hub_udp(hub->id)) {
         /* This search is not for our IP:port */
@@ -1845,6 +1879,11 @@ static void handle_connect(net_t *n, const char *addr) {
   if(!net_is_connected(hub->net))
     return;
 
+  /* If we have a pre-configured active IP, make sure to enable active mode
+   * immediately. */
+  if(hub_ip(hub))
+    listen_refresh();
+
   if(hub->adc)
     net_writef(hub->net, "HSUP ADBASE ADTIGR%s\n", var_get_bool(hub->id, VAR_adc_blom) ? " ADBLO0 ADBLOM" : "");
 
@@ -1860,7 +1899,7 @@ static void handle_connect(net_t *n, const char *addr) {
 void hub_connect(hub_t *hub) {
   char *oaddr = var_get(hub->id, VAR_hubaddr);
   yuri_t addr;
-  yuri_parse(oaddr, &addr);
+  yuri_parse_copy(oaddr, &addr);
   if(!addr.port)
     addr.port = 411;
   hub->adc = strncmp(addr.scheme, "adc", 3) == 0;
@@ -1877,6 +1916,7 @@ void hub_connect(hub_t *hub) {
 
   ui_mf(hub->tab, 0, "Connecting to %s...", oaddr);
   net_connect(hub->net, addr.host, addr.port, var_get(hub->id, VAR_local_address), handle_connect);
+  free(addr.buf);
 }
 
 
