@@ -1,6 +1,6 @@
 /* ncdc - NCurses Direct Connect client
 
-  Copyright (c) 2011-2013 Yoran Heling
+  Copyright (c) 2011-2014 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -78,6 +78,10 @@
 #include <ncurses.h>
 #endif
 
+#ifdef USE_GEOIP
+#include <GeoIP.h>
+#endif
+
 
 // GnuTLS / libgcrypt functions
 // crypt_aes128cbc() uses a 16-byte zero'd IV. Data is encrypted or decrypted in-place.
