@@ -1,6 +1,6 @@
 /* ncdc - NCurses Direct Connect client
 
-  Copyright (c) 2011-2013 Yoran Heling
+  Copyright (c) 2011-2014 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -114,6 +114,21 @@ struct net_t {
 
 
 
+static int net_sock_bind(int af, int sock, char *laddr) {
+  if(!laddr)
+    return 0;
+
+  if(af == AF_INET) {
+    struct in_addr a = var_parse_ip4(laddr);
+    return bind(sock, ip4_sockaddr(a, 0), sizeof(struct sockaddr_in));
+  }
+  if(af == AF_INET6) {
+    struct in6_addr a = var_parse_ip6(laddr);
+    return bind(sock, ip6_sockaddr(a, 0), sizeof(struct sockaddr_in6));
+  }
+  g_return_val_if_reached(0);
+}
+
 
 // Low-level recv/send wrappers
 
@@ -198,7 +213,7 @@ static int low_send(net_t *n, const char *buf, int len, const char **err) {
   if(n->state != NETST_DIS)
     time(&n->timeout_last);
   if(r < 0) {
-    *err = n->tls ? gnutls_strerror(errno) : g_strerror(errno);
+    *err = n->tls ? gnutls_strerror(r) : g_strerror(errno);
     return -1;
   }
 
@@ -363,7 +378,10 @@ static void syn_upload_sendfile(synfer_t *s, int sock, fadv_t *adv) {
     // No need for a lock here, we're not using the TLS session and socket fd's
     // are thread-safe. To some extent at least.
 #ifdef HAVE_LINUX_SENDFILE
-    ssize_t r = sendfile(sock, s->fd, &off, MIN(b, s->left));
+    // XXX: On 32bit Linux with musl, sendfile() may fail with EOVERFLOW when
+    // an offset argument is given and is larger than UINT32_MAX, so we're
+    // passing NULL instead to use the fd's internal file offset.
+    ssize_t r = sendfile(sock, s->fd, NULL, MIN(b, s->left));
 #elif HAVE_BSD_SENDFILE
     off_t len = 0;
     gint64 r = sendfile(s->fd, sock, off, (size_t)MIN(b, s->left), NULL, &len, 0);
@@ -388,9 +406,15 @@ static void syn_upload_sendfile(synfer_t *s, int sock, fadv_t *adv) {
       continue;
     } else if(errno == EAGAIN || errno == EINTR) {
       continue;
-    } else if(errno == ENOTSUP || errno == ENOSYS || errno == EINVAL) {
-      g_message("sendfile() failed with `%s', using fallback.", g_strerror(errno));
+    } else if(errno == ENOTSUP || errno == ENOSYS || errno == EINVAL || errno == EOVERFLOW) {
       // Don't set s->err here, let the fallback handle the rest
+      g_message("sendfile() failed with `%s', using fallback.", g_strerror(errno));
+      // The fallback code continues from the fd position, so make sure to
+      // update it in case (FreeBSD's) sendfile() didn't do so.
+      if(lseek(s->fd, off, SEEK_SET) == (off_t)-1) {
+        g_message("Can't switch to fallback, seek failed: %d (%s)", errno, g_strerror(errno));
+        s->err = g_strdup(g_strerror(errno));
+      }
       return;
     } else {
       if(errno != EPIPE && errno != ECONNRESET)
@@ -882,12 +906,10 @@ void net_write(net_t *n, const char *buf, int len) {
 // Logs the write for debugging. Does not log a trailing newline if there is one.
 static void asy_debugwrite(net_t *n, int oldlen) {
   if(n->wbuf->len && n->wbuf->len > oldlen) {
-    char end = n->wbuf->str[n->wbuf->len-1];
-    if(end == '\n')
-      n->wbuf->str[n->wbuf->len-1] = 0;
-    g_debug("%s> %s", net_remoteaddr(n), n->wbuf->str+oldlen);
-    if(end == '\n')
-      n->wbuf->str[n->wbuf->len-1] = end;
+    int len = n->wbuf->len-oldlen;
+    if(n->wbuf->str[n->wbuf->len-1] == '\n')
+      len--;
+    g_debug("%s> %.*s", net_remoteaddr(n), len, n->wbuf->str+oldlen);
   }
 }
 
@@ -1094,15 +1116,7 @@ static void dnscon_tryconn(net_t *n) {
   n->sock = socket(c->ai_family, SOCK_STREAM, 0);
   fcntl(n->sock, F_SETFL, fcntl(n->sock, F_GETFL, 0)|O_NONBLOCK);
 
-  int r = 0;
-  if(n->dnscon->laddr && c->ai_family == AF_INET) {
-    struct in_addr a = var_parse_ip4(n->dnscon->laddr);
-    r = bind(n->sock, ip4_sockaddr(a, 0), sizeof(struct sockaddr_in));
-  } else if(n->dnscon->laddr && c->ai_family == AF_INET6) {
-    struct in6_addr a = var_parse_ip6(n->dnscon->laddr);
-    r = bind(n->sock, ip6_sockaddr(a, 0), sizeof(struct sockaddr_in6));
-  }
-  if(r < 0) {
+  if(net_sock_bind(c->ai_family, n->sock, n->dnscon->laddr) < 0) {
     char *e = g_strdup_printf("Can't bind to local address: %s", g_strerror(errno));
     g_debug("%s: %s", net_remoteaddr(n), e);
     n->cb_err(n, NETERR_CONN, e);
@@ -1110,7 +1124,7 @@ static void dnscon_tryconn(net_t *n) {
     return;
   }
 
-  r = connect(n->sock, n->dnscon->next->ai_addr, n->dnscon->next->ai_addrlen);
+  int r = connect(n->sock, n->dnscon->next->ai_addr, n->dnscon->next->ai_addrlen);
 
   // The common case, I guess
   if(r && errno == EINPROGRESS) {
@@ -1220,7 +1234,7 @@ static gboolean handle_timer(gpointer dat) {
 
   // 30 second timeout on connecting, disconnecting, synchronous transfers, and
   // non-keepalive ASY connections.
-  if(intv > 30 && (n->state == NETST_DNS || n->state == NETST_CON || n->state == NETST_DIS || (n->state == NETST_ASY && !n->timeout_msg))) {
+  if(intv > 30 && (n->state == NETST_DNS || n->state == NETST_CON || n->state == NETST_DIS || n->state == NETST_SYN || (n->state == NETST_ASY && !n->timeout_msg))) {
     if(n->state == NETST_DNS || n->state == NETST_CON)
       n->cb_err(n, NETERR_TIMEOUT, g_strerror(ETIMEDOUT));
     else {
@@ -1293,7 +1307,8 @@ void net_disconnect(net_t *n) {
   case NETST_ASY:
   case NETST_DIS:
     n->rd_cb = NULL;
-    if(n->syn) {
+    s = n->syn;
+    if(s) {
       syn_cancel(n);
       syn_free(s);
     }
@@ -1369,114 +1384,92 @@ void net_unref(net_t *n) {
 
 
 
-// Some global stuff for sending UDP packets
+// Simple API for sending UDP packets
 
-typedef struct net_udp_t {
-  char host[62];
-  unsigned short port;
-  int sock; /* net_udp4_sock or net_udp6_sock */
-  char *msg;
-  int msglen;
-} net_udp_t;
+#if INTERFACE
 
-static int net_udp4_sock, net_udp6_sock;
-static GQueue *net_udp_queue;
+struct net_udp_t {
+  char addr[62];
+  int sock;
+};
+
+#endif
 
 
-static gboolean udp_handle_out(gpointer);
+// Creates a new UDP socket for sending messages to the given destination.
+// host is assumed to be a valid IPv4 or IPv6 address.
+void net_udp_init(net_udp_t *udp, const char *host, unsigned short port, char *laddr) {
+  int af = ip4_isvalid(host) ? AF_INET : AF_INET6;
+  snprintf(udp->addr, sizeof(udp->addr), af == AF_INET ? "%s:%d" : "[%s]:%d", host, (int)port);
 
-static void udp_setwatcher() {
-  net_udp_t *m = g_queue_peek_head(net_udp_queue);
-  if(!m)
+  udp->sock = socket(af, SOCK_DGRAM, 0);
+  fcntl(udp->sock, F_SETFL, fcntl(udp->sock, F_GETFL, 0)|O_NONBLOCK);
+
+  if(net_sock_bind(af, udp->sock, laddr) < 0) {
+    g_message("Can't bind UDP socket for '%s' to local address '%s': %s", udp->addr, laddr, g_strerror(errno));
+    close(udp->sock);
     return;
-  GSource *src = fdsrc_new(m->sock, G_IO_OUT);
-  g_source_set_callback(src, udp_handle_out, NULL, NULL);
-  g_source_attach(src, NULL);
-  g_source_unref(src);
-}
-
-
-static gboolean udp_handle_out(gpointer dat) {
-  net_udp_t *m = g_queue_pop_head(net_udp_queue);
-  if(!m)
-    return FALSE;
+  }
 
   int n;
-  if(yuri_validate_ipv4(m->host, strlen(m->host)) == 0) {
-    struct in_addr a = ip4_pack(m->host);
-    n = sendto(net_udp4_sock, m->msg, m->msglen, 0, ip4_sockaddr(a, m->port), sizeof(struct sockaddr_in));
+  if(af == AF_INET) {
+    struct in_addr a = ip4_pack(host);
+    n = connect(udp->sock, ip4_sockaddr(a, port), sizeof(struct sockaddr_in));
   } else {
-    struct in6_addr a = ip6_pack(m->host);
-    n = sendto(net_udp6_sock, m->msg, m->msglen, 0, ip6_sockaddr(a, m->port), sizeof(struct sockaddr_in6));
+    struct in6_addr a = ip6_pack(host);
+    n = connect(udp->sock, ip6_sockaddr(a, port), sizeof(struct sockaddr_in6));
   }
-
-  if(n == -1 && (errno == EWOULDBLOCK || errno == EAGAIN || errno == EINTR)) {
-    g_queue_push_head(net_udp_queue, m);
-    return TRUE;
-  }
-
-  if(n != m->msglen)
-    g_message("Error sending UDP message: %s.", g_strerror(errno));
-  else {
-    ratecalc_add(&net_out, m->msglen);
-  }
-  g_free(m->msg);
-  g_slice_free(net_udp_t, m);
-  udp_setwatcher();
-  return FALSE;
-}
-
-
-// host is assumed to be a valid IPv4 or IPv6 address.
-// TODO: Outgoing udp socket should be bound to the local_address config option!
-void net_udp_send_raw(const char *host, unsigned short port, const char *msg, int len) {
-  net_udp_t *m = g_slice_new0(net_udp_t);
-  m->msg = g_memdup(msg, len);
-  m->msglen = len;
-  m->port = port;
-  strncpy(m->host, host, sizeof(m->host));
-  m->sock = yuri_validate_ipv4(host, strlen(host)) == 0 ? net_udp4_sock : net_udp6_sock;
-
-  g_queue_push_tail(net_udp_queue, m);
-  if(net_udp_queue->head == net_udp_queue->tail)
-    udp_setwatcher();
-}
-
-
-static void net_udp_debug() {
-  if(net_udp_queue->tail) {
-    net_udp_t *m = net_udp_queue->tail->data;
-    char end = m->msglen > 0 ? m->msg[m->msglen-1] : 0;
-    if(end == '\n')
-      m->msg[m->msglen-1] = 0;
-    g_debug("UDP:%s:%d> %s", m->host, (int)m->port, m->msg);
-    if(end == '\n')
-      m->msg[m->msglen-1] = end;
+  if(n < 0) {
+    g_message("Can't associate UDP socket with '%s': %s", udp->addr, g_strerror(errno));
+    close(udp->sock);
+    return;
   }
 }
 
 
-void net_udp_send(const char *host, unsigned short port, const char *msg) {
-  net_udp_send_raw(host, port, msg, strlen(msg));
-  net_udp_debug();
+void net_udp_destroy(net_udp_t *udp) {
+  if(udp->sock >= 0)
+    close(udp->sock);
 }
 
 
-void net_udp_sendf(const char *host, unsigned short port, const char *fmt, ...) {
+// Send a message to a UDP socket, logs but otherwise ignores errors. Note that
+// the socket is created as non-blocking and this function does not attempt to
+// retry the send() on EWOULDBLOCK or EAGAIN. It is assumed that the kernel
+// buffers are large enough that we can burst-queue several messages, and that,
+// if the kernel buffers are full, we might be better off dropping some
+// messages than queueing them until infinity.
+void net_udp_send_raw(net_udp_t *udp, const char *msg, int len) {
+  int r = send(udp->sock, msg, len, 0);
+  if(r != len)
+    g_message("Error sending UDP message to '%s': %s", udp->addr, g_strerror(errno));
+  else
+    ratecalc_add(&net_out, len);
+}
+
+
+void net_udp_send(net_udp_t *udp, const char *msg) {
+  int len = strlen(msg);
+  if(len) {
+    g_debug("UDP:%s> %.*s", udp->addr, msg[len-1] == '\n' ? len-1 : len, msg);
+    net_udp_send_raw(udp, msg, len);
+  }
+}
+
+
+void net_udp_sendf(net_udp_t *udp, const char *fmt, ...) {
   va_list va;
   va_start(va, fmt);
   char *str = g_strdup_vprintf(fmt, va);
   va_end(va);
-  net_udp_send_raw(host, port, str, strlen(str));
+  net_udp_send(udp, str);
   g_free(str);
-  net_udp_debug();
 }
 
 
 
 
 
-
 // initialize some global structures
 
 void net_init_global() {
@@ -1488,11 +1481,5 @@ void net_init_global() {
 
   dns_pool = g_thread_pool_new(dnscon_thread, NULL, -1, FALSE, NULL);
   syn_pool = g_thread_pool_new(syn_thread, NULL, -1, FALSE, NULL);
-
-  net_udp4_sock = socket(AF_INET, SOCK_DGRAM, 0);
-  net_udp6_sock = socket(AF_INET6, SOCK_DGRAM, 0);
-  fcntl(net_udp4_sock, F_SETFL, fcntl(net_udp4_sock, F_GETFL, 0)|O_NONBLOCK);
-  fcntl(net_udp6_sock, F_SETFL, fcntl(net_udp6_sock, F_GETFL, 0)|O_NONBLOCK);
-  net_udp_queue = g_queue_new();
 }
 
