@@ -1,6 +1,6 @@
 /* ncdc - NCurses Direct Connect client
 
-  Copyright (c) 2011-2013 Yoran Heling
+  Copyright (c) 2011-2014 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -82,7 +82,7 @@ ui_tab_t *uit_conn_create() {
   t->s_conn = t->s_idle = t->s_upload = t->s_download = t->s_discon = TRUE;
   // sort the connection list
   g_sequence_sort(cc_list, sort_func, NULL);
-  t->list = ui_listing_create(cc_list, skip_func, t);
+  t->list = ui_listing_create(cc_list, skip_func, t, NULL);
   return (ui_tab_t *)t;
 }
 
@@ -154,7 +154,7 @@ static void t_draw_row(ui_listing_t *list, GSequenceIter *iter, int row, void *d
       file++;
     else
       file = cc->last_file;
-      mvaddnstr(row, 58, file, str_offset_from_columns(file, wincols-58));
+    mvaddnstr(row, 58, file, str_offset_from_columns(file, wincols-58));
   }
 
   attroff(iter == list->sel ? UIC(list_select) : UIC(list_default));
@@ -246,7 +246,8 @@ static void t_draw(ui_tab_t *tab) {
   attroff(UIC(list_header));
 
   int bottom = t->details ? winrows-11 : winrows-3;
-  ui_listing_draw(t->list, 2, bottom-1, t_draw_row);
+  ui_cursor_t cursor;
+  ui_listing_draw(t->list, 2, bottom-1, &cursor, t_draw_row);
 
   // footer
   attron(UIC(separator));
@@ -272,6 +273,7 @@ static void t_draw(ui_tab_t *tab) {
   // detailed info
   if(t->details)
     t_draw_details(t, bottom);
+  move(cursor.y, cursor.x);
 }
 
 
