@@ -1,6 +1,6 @@
 /* ncdc - NCurses Direct Connect client
 
-  Copyright (c) 2011-2013 Yoran Heling
+  Copyright (c) 2011-2014 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -87,7 +87,7 @@ static void setusers(dl_t *dl) {
   int i;
   for(i=0; i<dl->u->len; i++)
     g_sequence_insert_sorted(l, g_sequence_get(g_ptr_array_index(dl->u, i)), dud_sort_func, NULL);
-  dltab->users = ui_listing_create(l, NULL, NULL);
+  dltab->users = ui_listing_create(l, NULL, NULL, NULL);
   dltab->cur = dl;
 }
 
@@ -106,7 +106,7 @@ ui_tab_t *uit_dl_create() {
   dl_t *dl;
   while(g_hash_table_iter_next(&iter, NULL, (gpointer *)&dl))
     dl->iter = g_sequence_insert_sorted(l, dl, sort_func, NULL);
-  dltab->list = ui_listing_create(l, NULL, NULL);
+  dltab->list = ui_listing_create(l, NULL, NULL, NULL);
 
   return (ui_tab_t *)dltab;
 }
@@ -213,7 +213,8 @@ static void t_draw(ui_tab_t *tab) {
   attroff(UIC(list_header));
 
   int bottom = dltab->details ? winrows-14 : winrows-4;
-  int pos = ui_listing_draw(dltab->list, 2, bottom-1, draw_row);
+  ui_cursor_t cursor;
+  int pos = ui_listing_draw(dltab->list, 2, bottom-1, &cursor, draw_row);
 
   dl_t *sel = g_sequence_iter_is_end(dltab->list->sel) ? NULL : g_sequence_get(dltab->list->sel);
 
@@ -244,8 +245,9 @@ static void t_draw(ui_tab_t *tab) {
     if(!dltab->users || !g_sequence_get_length(dltab->users->list))
       mvaddstr(bottom+3, 0, "  No users for this download.");
     else
-      ui_listing_draw(dltab->users, bottom+2, winrows-3, dud_draw_row);
+      ui_listing_draw(dltab->users, bottom+2, winrows-3, &cursor, dud_draw_row);
   }
+  move(cursor.y, cursor.x);
 }
 
 
@@ -301,9 +303,10 @@ static void t_key(ui_tab_t *tab, guint64 key) {
     }
     break;
   case INPT_CHAR('c'): // c - find connection
+    /* XXX: If the file is downloaded from multiple users, this'll only select one (obviously) */
     if(!sel)
       ui_m(NULL, 0, "Nothing selected.");
-    else if(!sel->active)
+    else if(!sel->active_threads)
       ui_m(NULL, 0, "Download not in progress.");
     else {
       cc_t *cc = NULL;
