@@ -1,6 +1,6 @@
 /* ncdc - NCurses Direct Connect client
 
-  Copyright (c) 2011-2013 Yoran Heling
+  Copyright (c) 2011-2014 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -141,7 +141,7 @@ static void t_draw(ui_tab_t *tab) {
   mvaddstr(winrows-3, 0, tab->name);
   addstr("> ");
   int pos = str_columns(tab->name)+2;
-  ui_textinput_draw(ui_global_textinput, winrows-3, pos, wincols-pos);
+  ui_textinput_draw(ui_global_textinput, winrows-3, pos, wincols-pos, NULL);
 }
 
 
