@@ -1,6 +1,6 @@
 /* ncdc - NCurses Direct Connect client
 
-  Copyright (c) 2011-2013 Yoran Heling
+  Copyright (c) 2011-2014 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -61,7 +61,7 @@ static void t_draw(ui_tab_t *t) {
   ui_logwindow_draw(t->log, 1, 0, winrows-4, wincols);
 
   mvaddstr(winrows-3, 0, "main>");
-  ui_textinput_draw(ui_global_textinput, winrows-3, 6, wincols-6);
+  ui_textinput_draw(ui_global_textinput, winrows-3, 6, wincols-6, NULL);
 }
 
 
