@@ -1,6 +1,6 @@
 /* ncdc - NCurses Direct Connect client
 
-  Copyright (c) 2011-2013 Yoran Heling
+  Copyright (c) 2011-2014 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -467,8 +467,8 @@ gboolean file_move(const char *from, const char *to, gboolean overwrite, GError
     if(r <= 0)
       break;
 
+    char *p = buf;
     while(r > 0) {
-      char *p = buf;
       int w = write(tofd, p, r);
       if(w < 0 && errno == EINTR)
         continue;
@@ -497,6 +497,34 @@ err:
 
 
 
+// Bit array utility functions. These functions work on an array of guint8,
+// where each bit can be accessed with its own index.
+// This does not use, because those may require byte swapping to store in a
+// endian-neutral way. These bit arrays are naturally endian-neutral, so can be
+// stored and retreived easily.
+
+#if INTERFACE
+
+#define bita_size(n) ((n+7)/8)
+
+#define bita_new(n) g_new0(guint8, bita_size(n))
+
+#define bita_free(a) g_free(a)
+
+#define bita_get(a, i) (((a)[i/8] >> (i&7)) & 1)
+
+#define bita_set(a, i) ((a)[i/8] |= 1<<(i&7))
+
+#define bita_reset(a, i) ((a)[i/8] &= ~(1<<(i&7)))
+
+#define bita_val(a, i, t) (t ? bita_set(a, i) : bita_reset(a, i))
+
+#endif
+
+
+
+
+
 #if INTERFACE
 
 // Tests whether a string is a valid base32-encoded string.
@@ -506,6 +534,12 @@ err:
 // base32-encoded 39-character string.
 #define istth(s) (strlen(s) == 39 && isbase32(s))
 
+#define MAXCIDLEN 64 /* 512 bits */
+
+// Test whether a string is a valid CID. I.e. a base32-encoded string between
+// 128 and MAXCIDLEN bits */
+#define iscid(s) (strlen(s) >= 26 && strlen(s) <= MAXCIDLEN*8/5+1 && isbase32(s))
+
 #endif
 
 
@@ -555,6 +589,16 @@ void base32_decode(const char *from, char *to) {
 struct in_addr ip4_any = {};
 struct in6_addr ip6_any = {};
 
+gboolean ip4_isvalid(const char *str) {
+  struct in_addr a;
+  return inet_pton(AF_INET, str, &a) == 1;
+}
+
+gboolean ip6_isvalid(const char *str) {
+  struct in6_addr a;
+  return inet_pton(AF_INET6, str, &a) == 1;
+}
+
 struct in_addr ip4_pack(const char *str) {
   struct in_addr a;
   return !str || inet_pton(AF_INET, str, &a) != 1 ? ip4_any : a;
@@ -1014,6 +1058,7 @@ struct fadv_t {
 #define fadv_init(a,f,o,l) ((a)->fd = 0)
 #define fadv_purge(a, l)   ((a)->fd = 0)
 #define fadv_close(a)      ((a)->fd = 0)
+#define fadv_oneshot(f,o,s,l)
 
 #endif
 
@@ -1034,6 +1079,18 @@ void fadv_purge(fadv_t *a, int length) {
   }
 }
 
+/* A one-shot function to flush all pages that the given range resides on,
+ * except for the last page if the range isn't page-aligned. */
+void fadv_oneshot(int fd, guint64 off, size_t length, int flag) {
+  if(!(var_ffc_get() & flag))
+    return;
+  length += off & 511;
+  off &= ~511;
+  length &= ~511;
+  if(length != off)
+    posix_fadvise(fd, off, length, POSIX_FADV_DONTNEED);
+}
+
 #endif
 
 
