@@ -1,6 +1,6 @@
 /* ncdc - NCurses Direct Connect client
 
-  Copyright (c) 2011-2013 Yoran Heling
+  Copyright (c) 2011-2014 Yoran Heling
 
   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
@@ -119,9 +119,9 @@ static char *p_ip(const char *val, GError **err) {
   struct in6_addr i6 = ip6_any;
   char *sep = strchr(val, ',');
   if(sep) *sep = 0;
-  if(yuri_validate_ipv4(val, strlen(val)) == 0)
+  if(ip4_isvalid(val))
     i4 = ip4_pack(val);
-  else if(yuri_validate_ipv6(val, strlen(val)) == 0)
+  else if(ip6_isvalid(val))
     i6 = ip6_pack(val);
   else {
     g_set_error_literal(err, 1, 0, "Invalid IP.");
@@ -131,9 +131,9 @@ static char *p_ip(const char *val, GError **err) {
     *(sep++) = ',';
     while(*sep == ' ')
       sep++;
-    if(ip4_isany(i4) && yuri_validate_ipv4(sep, strlen(sep)) == 0)
+    if(ip4_isany(i4) && ip4_isvalid(sep))
       i4 = ip4_pack(sep);
-    else if(ip6_isany(i6) && yuri_validate_ipv6(sep, strlen(sep)) == 0)
+    else if(ip6_isany(i6) && ip6_isvalid(sep))
       i6 = ip6_pack(sep);
     else {
       g_set_error_literal(err, 1, 0, "Invalid IP.");
@@ -308,6 +308,24 @@ static char *p_backlog(const char *val, GError **err) {
 }
 
 
+// download_segment
+
+static char *f_download_segment(const char *var) {
+  return g_strdup(strcmp(var, "0") == 0 ? "0 (disable segmented downloading)" : str_formatsize(int_raw(var)));
+}
+
+static char *p_download_segment(const char *val, GError **err) {
+  guint64 size = str_parsesize(val);
+  if(size == G_MAXUINT64) {
+    g_set_error_literal(err, 1, 0, "Invalid speed.");
+    return NULL;
+  }
+  if(size && size < DLFILE_CHUNKSIZE)
+    size = DLFILE_CHUNKSIZE;
+  return g_strdup_printf("%"G_GUINT64_FORMAT, size);
+}
+
+
 // nick
 
 static char *p_nick(const char *val, GError **err) {
@@ -922,6 +940,7 @@ struct var_t {
   V(download_dir,     1,0, f_id,           p_id,            su_path,       NULL,         s_dl_inc_dir,    i_dl_inc_dir(TRUE))\
   V(download_exclude, 1,0, f_id,           p_regex,         su_old,        NULL,         NULL,            NULL)\
   V(download_rate,    1,0, f_speed,        p_speed,         NULL,          NULL,         NULL,            NULL)\
+  V(download_segment, 1,0, f_download_segment,p_download_segment,NULL,     NULL,         NULL,            g_strdup_printf("%"G_GUINT64_FORMAT, (guint64)DLFILE_CHUNKSIZE))\
   V(download_slots,   1,0, f_int,          p_int,           NULL,          NULL,         s_download_slots,"3")\
   V(email,            1,1, f_id,           p_id,            su_old,        NULL,         s_hubinfo,       NULL)\
   V(encoding,         1,1, f_id,           p_encoding,      su_encoding,   NULL,         NULL,            "UTF-8")\
@@ -1033,6 +1052,11 @@ int var_get_int(guint64 h, var_type n) {
   return int_raw(r);
 }
 
+gint64 var_get_int64(guint64 h, var_type n) {
+  char *r = var_get(h, n);
+  return int_raw(r);
+}
+
 // Extract the IPv4 address from a p_ip()-formatted config option.
 struct in_addr var_parse_ip4(char *r) {
   struct in_addr a = ip4_any;
